/*
 * clock for root window.
 */

/*
 * - finish alarms set up
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "config.h"
#include "options.h"
#include "rootapp.h"

#define EVENTMASK VisibilityChangeMask | ExposureMask

/* Global varriables */
static int end_program = FALSE;

static char       timestr[MAXSTRLEN + 1];
static char       datestr[MAXSTRLEN + 1];
static char      *dpyname    = NULL;
static char      *fontname   = NULL;
static char      *geometry   = NULL;
static char      *dgeometry  = NULL;
static char      *colorname  = NULL;
static StrLst    *alarms     = NULL;
static char      *alarm_cmd  = NULL;
static char      *dformat    = NULL;


/* Command line options and config file entries */
static Option opttab[] = {
    {'h', NULL,       NULL, O_HELP,   NULL,       FALSE, "display help"},
    {'v', NULL,       NULL, O_VERS,   NULL,       FALSE, "display version"},
    {'d', "display",  NULL, C_STRING, &dpyname,   FALSE, "display name"},
    {'c', "color",    NULL, C_STRING, &colorname, FALSE, "text color"},
    {'f', "font",     NULL, C_STRING, &fontname,  FALSE, "font to use"},
    {'g', "geom",     NULL, C_STRING, &geometry,  FALSE, "set clock geometry"},
    {'a', "alarm",    NULL, C_STRLST, &alarms,    FALSE, "alarm time"},
    {'x', "cmd",      NULL, C_STRING, &alarm_cmd, FALSE, "alarm play command"},
    {'z', "dategeom", NULL, C_STRING, &dgeometry, FALSE, "set date geometry"},
    {'u', "datefmt",  NULL, C_STRING, &dformat,   FALSE, "set date format"},
    {0,   NULL,       NULL, C_NONE,   NULL,       FALSE, NULL}
};



/****** UPDATING FUNCTIONS ******/

int update_now() {
    time_t     tnow;
    struct tm *now;
    int        ret     = 0;
    char      *datefmt = dformat ? dformat : "%A %d %B";

    time(&tnow);
    now = localtime(&tnow);
    bzero(timestr, MAXSTRLEN + 1);
    bzero(datestr, MAXSTRLEN + 1);
    strftime(timestr, MAXSTRLEN, "%T", now);
    strftime(datestr, MAXSTRLEN, datefmt, now);
    if ((now->tm_hour == 0) && (now->tm_min ==0) && (now->tm_sec == 0))
        ret = 1;
    if ((now->tm_sec == 0) && alarm_cmd && alarms) {
        char    thistime[MAXSTRLEN + 1];
        StrLst *alarm   = alarms;
        int     execute = FALSE;

        strftime(thistime, MAXSTRLEN, "%H:%M", now);
        while (alarm) {
            if (strcmp(thistime, alarm->entry) == 0) execute = TRUE;
            alarm = alarm->next;
        }
        if (execute) my_system(alarm_cmd, NULL);
    }
    return ret;
}


/****** PROCESSING FUNCTIONS ******/

void display_data(RootApp *app, const char *text, int y) {
    if (app) {
        clear_bg(app);
        root_print(app, text, 0, y);
        display_app(app);
    }
}


void main_loop(RootApp *t, RootApp *d) {
    XEvent event;
    int cont = TRUE;

    display_data(t, timestr, 1);
    display_data(d, datestr, 0);
    while (cont) {
        nextevent_or_timeout(&event, 1000);
        if ((update_now() == 1) && d)
            set_to_text_size(d, dgeometry, datestr);
        display_data(t, timestr, 1);
        display_data(d, datestr, 0);
        if (end_program) cont = FALSE;
    }
}


/****** SIGNALS ******/

void install_handler();


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void install_handler() {
    signal(SIGTERM, signals_handler);
    signal(SIGQUIT, signals_handler);
    signal(SIGINT,  signals_handler);
}


void manage_signals() {
    struct sigaction sa;

    sa.sa_handler = SIG_IGN;
#ifdef SA_NOCLDWAIT
    sa.sa_flags = SA_NOCLDWAIT;
#else
    sa.sa_flags = 0;
#endif
    sigemptyset(&sa.sa_mask);
    sigaction(SIGCHLD, &sa, NULL);
    install_handler();
}


/****** MAIN ******/


int main(int argc, char *argv[]) {
    RootApp *rtime = NULL;
    RootApp *rdate = NULL;

    /* General init */
    manage_signals();
    setlocale(LC_ALL, "");
    parse_argv(opttab, argc, argv);
    update_now();

    /* X11 init */
    open_x11(dpyname);
    if (fontname) load_font(fontname);
    if (colorname) set_color_name(colorname);

    /* Creation */
    rtime = create_app(EVENTMASK, geometry);
    set_to_text_size(rtime, geometry, timestr);
    if (dgeometry) {
        rdate = create_app(EVENTMASK, dgeometry);
        set_to_text_size(rdate, dgeometry, datestr);
    }

    main_loop(rtime, rdate);

    /* Clean */
    close_app(rtime);
    if (rdate) close_app(rdate);
    close_x11();
    free_vars(opttab);

    return 0;
}
